/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
 package org.objectweb.speedo.api;

import java.util.List;
import java.util.Arrays;
import java.util.Collections;

/**
 * Defines the Speedo options which can be passed as properties to the
 * PersistenceManagerFactory
 */
public interface SpeedoProperties {

    String LOGGER_NAME = "org.objectweb.speedo";

	String DEBUG = "org.objectweb.speedo.debug"; 

    // Constant usable in the Properties of a PersistenceManagerFactory

    /**
     * Is the property value to pass to the PersistenceManagerFactory to
     * indicate that the size is unlimited.
     */
    String NO_LIMIT = "nolimit";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * cache size.
     */
    String CACHE_SIZE = "org.objectweb.speedo.cache.size";

	String CACHE_AUTO_CLEAN_SIZE = "org.objectweb.speedo.cache.autoCleanSize";
	String CACHE_AUTO_CLEAN_THRESHOLD = "org.objectweb.speedo.cache.autoCleanThreshold";

	String CACHE_REPLCAEMENT = "org.objectweb.speedo.cache.policy";
	String CACHE_REPLCAEMENT_FIFO = "FIFO";
	String CACHE_REPLCAEMENT_LRU = "LRU";
	String CACHE_REPLCAEMENT_MRU = "MRU";

    String CACHE_CLASS_POLICY = "org.objectweb.speedo.cache.classPolicy";
    String CACHE_CLASS_POLICY_FIXED = "fixed";
    String CACHE_CLASS_POLICY_WITH_CACHE = "cache";
    String CACHE_CLASS_POLICY_NOCACHE = "nocache";
    
    /**
     * Is the property to pass to the PersistenceManagerFactory to indicate
     * if the transactionnal environnement is managed.
     */
    String MANAGED = "org.objectweb.perseus.connector.ra.jdo.managed";

	String TM_NAME = "org.objectweb.speedo.jca.TMName";
	/**
	 * @deprecated use TM_NAME
	 */
	String TM_NAME_old = "org.objectweb.perseus.connector.ra.jdo.TMName";
    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * maximum size of PersistenceManager pool.
     */
    String PM_POOL_MIN = "org.objectweb.speedo.persistencemanager.pool.min";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * minimum size of PersistenceManager pool.
     */
    String PM_POOL_MAX = "org.objectweb.speedo.persistencemanager.pool.max";
    String PM_POOL_TTL = "org.objectweb.speedo.persistencemanager.pool.ttl";
    String PM_POOL_TIMEOUT = "org.objectweb.speedo.persistencemanager.pool.timeout";

    /**
     * Is the property to pass to the PersistenceManagerFactory to assign the
     * minimum size of compiled query pool.
     */
    String COMPILED_QUERY_CACHE_SIZE = "org.objectweb.speedo.compiledquery.cache.size";
	String COMPILED_QUERY_CACHE_AUTO_CLEAN_SIZE = "org.objectweb.speedo.compiledquery.cache.autoCleanSize";
	String COMPILED_QUERY_CACHE_AUTO_CLEAN_THRESHOLD = "org.objectweb.speedo.compiledquery.cache.autoCleanThreshold";
    String COMPILED_QUERY_CACHE_POLICY = "org.objectweb.speedo.compiledquery.cache.policy";

    /**
     * Is the property to pass to the PersistenceManagerFactory to specify if
     * the query must prefetch the data. The default value is true.
     */
    String PREFETCH = "org.objectweb.speedo.query.prefetch";
    String PREFETCH_ON_QUERY = "org.objectweb.speedo.query.prefetch.query";
    String PREFETCH_ON_EXTENT = "org.objectweb.speedo.query.prefetch.extent";

    /**
     * Is the property to pass to the PersistenceManagerFactory to specify the
	 * mapping structure management:
	 * - DO_NOTHING
	 * - CREATE_IF_REQUIRED
	 * - FORCE_CREATE
     */
    String MAPPING_STRUCTURE = "org.objectweb.speedo.mappingStructure";
    String MAPPING_STRUCTURE_DN = "DO_NOTHING";
    String MAPPING_STRUCTURE_CIR = "CREATE_IF_REQUIRED";
    String MAPPING_STRUCTURE_FC = "FORCE_CREATE";
    String MAPPING_STRUCTURE_DD = "DELETE_DATA";

	/**
	 * Is the property to pass to the PersistenceManagerFactory to specify if
	 * the query must prefetch the data. The default value is true.
	 */
	String TX_LISTENER = "org.objectweb.speedo.txListener";
	String CONNECTION_POOL_MIN = "org.objectweb.speedo.connection.pool.min";
	String CONNECTION_POOL_MAX = "org.objectweb.speedo.connection.pool.max";
	String CONNECTION_POOL_TTL = "org.objectweb.speedo.connection.pool.ttl";
	String CONNECTION_POOL_TIMEOUT = "org.objectweb.speedo.connection.pool.timeout";


    // Constant usable in the .jdo files (keys of the extension tags)

    String VENDOR_NAME = "speedo";

    String ID = "id";
    String ID_SEQUENCE = "sequence";
    String ID_POLYMORPH_2L = "polymorph2l";
    String ID_LONG = "long";

    /**
     * is a string used in an extension tag in a.jdo file to specify
     * the sql column for a primitive field (not a reference).
     */
    String SQL_NAME = "sql-name";

	/**
	 * the INHERITANCE_MAPPING property permits to specifies the mapping type
	 * concerning the inheritance aspect.
	 */
	String INHERITANCE_MAPPING 				= "inheritance-mapping";
	String INHERITANCE_FILTERED_MAPPING 	= "filtered";
	String INHERITANCE_HORIZONTAL_MAPPING 	= "horizontal";
	String INHERITANCE_VERTICAL_MAPPING 	= "vertical";

	String INHERITANCE_FILTER 	            = "inheritance-filter";
	String INHERITANCE_KEY 	            	= "inheritance-key";

    /**
     * is a string used in an extension tag in a.jdo file to specify
     * the name of the sequence used for the datastore id.
     */
    String SQL_SEQ_NAME 		= "sql-seq-name";
    String SQL_SEQ_START 		= "sql-seq-start";
    String SQL_SEQ_INC 			= "sql-seq-increment";
    String SQL_SEQ_CACHE 		= "sql-seq-cache";
    String SQL_SEQ_ALLOCATOR	= "sql-seq-allocator";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the sql type for a primitive field (not a reference).
     */
    String SQL_TYPE = "sql-type";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the size of primitive field (not a reference).
     */
    String SIZE = "size";

	/**
	 * is a string to used in an extension tag in a.jdo file to specify
	 * the scale of primitive field (not a reference).
	 */
    String SCALE = "scale";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the foreign keys referencing the current persistent object.
     */
    String SOURCE_FK = "source-foreign-keys";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the foreign keys referencing the referenced persistent object.
     */
    String TARGET_FK = "target-foreign-keys";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the name of the reverse field.
     */
    String REVERSE_FIELD = "reverse-field";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the if persistent instance referenced by a field, must be removed when
     * the current class is deleted.
     */
    String CASCADE_DELETE = "cascade-delete";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the name of the join table in a multivalued reference (Collection, Map,
     * Array, Set, List).
     */
    String JOIN_TABLE = "join-table";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the name of the class permiting the conversion of a field between the
     * memory and the data support.
     */
    String FIELD_CONVERTER = "field-converter";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the column name of the index of the list or of the key in a map.
     */
    String INDEX = "index-sql-name";

    String INDEX_TYPE = "index-sql-type";

    /**
     * is a string to used in an extension tag in a.jdo file to specify
     * the column name of the element of the list or of the value in a map,
	 * when the element is a prmitive.
     */
    String ELEMENT = "element-sql-name";

    String ELEMENT_TYPE = "element-sql-type";
    
    String KEY_FIELD = "key-field";

    /**
     * Optionals features supported by JDO implementation (see supportedOptions();)
     */   
    String JDO_OPTION_TRANSIENT_TRANSACTIONAL = "javax.jdo.option.TransientTransactional"; 
    String JDO_OPTION_NON_TRANSACTIONAL_READ = "javax.jdo.option.NontransactionalRead";
    String JDO_OPTION_NON_TRANSACTIONAL_WRITE = "javax.jdo.option.NontransactionalWrite"; 
    String JDO_OPTION_RETAIN_VALUES = "javax.jdo.option.RetainValues";
    String JDO_OPTION_OPTIMISTIC = "javax.jdo.option.Optimistic";
	String JDO_OPTION_APPLICATION_IDENTITY = "javax.jdo.option.ApplicationIdentity"; 
	String JDO_OPTION_DATASTORE_IDENTITY = "javax.jdo.option.DatastoreIdentity";
	String JDO_OPTION_NON_DURABLE_IDENTITY = "javax.jdo.option.NonDurableIdentity";
	String JDO_OPTION_ARRAY_LIST = "javax.jdo.option.ArrayList";
	String JDO_OPTION_HASH_MAP = "javax.jdo.option.HashMap";
	String JDO_OPTION_HASH_TABLE = "javax.jdo.option.Hashtable"; 
	String JDO_OPTION_LINKED_LIST = "javax.jdo.option.LinkedList"; 
	String JDO_OPTION_TREE_MAP = "javax.jdo.option.TreeMap";
	String JDO_OPTION_TREE_SET = "javax.jdo.option.TreeSet"; 
	String JDO_OPTION_VECTOR = "javax.jdo.option.Vector";
	String JDO_OPTION_MAP = "javax.jdo.option.Map";
	String JDO_OPTION_LIST= "javax.jdo.option.List";	
	String JDO_OPTION_ARRAY = "javax.jdo.option.Array";
	String JDO_OPTION_NULL_COLLECTION = "javax.jdo.option.NullCollection";
	String JDO_OPTION_CHANGE_APPLICATION_IDENTITY = "javax.jdo.option.ChangeApplicationIdentity";
	String JDO_QUERY_JDOQL = "javax.jdo.query.JDOQL";
	
		
	/**
	 * Key values for Properties configuration options
	 */   
	String JDO_PERSISTENCE_MANAGER_FACTORY_CLASS = "javax.jdo.PersistenceManagerFactoryClass";
	String JDO_OPTION_RESTORE_VALUES =  "javax.jdo.option.RestoreValues"; 
	String JDO_OPTION_IGNORE_CACHE = "javax.jdo.option.IgnoreCache";
	String JDO_OPTION_MULTITREADED = "javax.jdo.option.Multithreaded";
	String JDO_OPTION_CONNECTION_DRIVER_NAME_old = "javax.jdo.option.DriverClassName";
	String JDO_OPTION_CONNECTION_DRIVER_NAME = "javax.jdo.option.DriverName";	
	String JDO_OPTION_CONNECTION_USER_NAME = "javax.jdo.option.ConnectionUserName";
	String JDO_OPTION_CONNECTION_PASSWORD = "javax.jdo.option.ConnectionPassword";
	String JDO_OPTION_CONNECTION_URL = "javax.jdo.option.ConnectionURL";
	String JDO_OPTION_CONNECTION_FACTORY_NAME = "javax.jdo.option.ConnectionFactoryName";
	String JDO_OPTION_CONNECTION_FACTORY2_NAME = "javax.jdo.option.ConnectionFactor2yName";
	String JDO_OPTION_MAPPING = "javax.jdo.option.ConnectionUserName";
	
	
	String MAPPER_NAME = "org.objectweb.speedo.mapperName";
	String TRANSACTION_LOCKING = "org.objectweb.speedo.transaction.locking";
	String SPEEDO_TRANSACTION_LOCKING = "speedo";
	String DB_TRANSACTION_LOCKING = "database";
	
	public final static int EXTENSION_JDO_IDX 		= 0;
	public final static int EXTENSION_PACKAGE_IDX 		= 1;
	public final static int EXTENSION_CLASS_IDX 		= 2;
	public final static int EXTENSION_FIELD_IDX 		= 3;
	public final static int EXTENSION_COLLECTION_IDX 	= 4;
	public final static int EXTENSION_MAP_IDX 		= 5;

	/**
	 * Defines the supported speedo extentsion on each xml node of the .jdo file
	 */
	public final static List SUPPORTED_EXTENSION = Arrays.asList(new List[]{
		Collections.EMPTY_LIST, //jdo
		Collections.EMPTY_LIST, //package
		Arrays.asList(new String[]{SQL_NAME,
								   SQL_SEQ_CACHE,
								   SQL_SEQ_INC,
								   SQL_SEQ_NAME,
								   SQL_SEQ_START,
								   SQL_SEQ_ALLOCATOR,
								   ID,
								   INHERITANCE_MAPPING,
								   INHERITANCE_KEY,
								   INHERITANCE_FILTER}), //class
		Arrays.asList(new String[]{SQL_NAME,
								   SQL_TYPE,
								   SIZE,
								   SOURCE_FK,
								   TARGET_FK,
								   JOIN_TABLE,
								   REVERSE_FIELD,
								   ELEMENT,
								   ELEMENT_TYPE,
								   INDEX,
								   INDEX_TYPE,
								   FIELD_CONVERTER,
								   CASCADE_DELETE,
								   KEY_FIELD}),//field
		Arrays.asList(new String[]{ELEMENT,
								   ELEMENT_TYPE,
								   INDEX,
								   INDEX_TYPE}),//collection
		Arrays.asList(new String[]{ELEMENT,
								   ELEMENT_TYPE,
								   INDEX,
								   INDEX_TYPE,
								   KEY_FIELD}),//map
	});
}
