/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass;

import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.speedo.mim.api.SpeedoAccessor;

import javax.jdo.PersistenceManager;

/**
 * Represents a element of a gen class. It matches to a reference element or
 * a primitive element.
 *
 * @author S.Chassande-Barrioz
 */
public interface GenClassElement extends PIndexedElem {

    /**
     * @return the index of this gen class element, possibly wrapped in an object.
     */
    Object getIndex();

    /**
     * Assignes the index of the element of the gen class. It is a user object.
     * @param index to add, it cannot be null.
     */
    void setIndex (Object index);

    /**
     * @return the element of the gen class. It is a user object.
     */
    Object getElement();

    /**
     * Assignes the element of the gen class. It is a user object.
     * @param element to add, it cannot be null.
     */
    void setElement(Object element);

    /**
     * @param pm is the persistence manager which permits to resolve the PName
     * into a java reference.
     * @return the element of the gen class. The element is a reference
     * (SpeedoProxy).
     */
    Object getElement(PersistenceManager pm);

    /**
     * Assignes the jorm status of the PIndexedElement
     * @param s the new status
     */
    void setStatus(byte s);


    /**
     * Retrieves the Speedo accessor associated to this gen class element.
     */
    SpeedoAccessor getSpeedoAccessor();

	void unSwizzle();
}
