/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass;

import org.objectweb.jorm.api.PAccessor;
import org.objectweb.jorm.api.PBinding;
import org.objectweb.jorm.api.PClassMapping;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.api.PMapper;
import org.objectweb.jorm.api.PMappingCallback;
import org.objectweb.jorm.api.PNameIterator;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.util.api.Loggable;
import org.objectweb.perseus.persistence.api.TransactionalPersistenceManager;
import org.objectweb.speedo.mim.lib.SpeedoHomeImpl;
import org.objectweb.speedo.pm.api.ProxyManagerFactory;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;

/**
 * This implementation of SpeedoHome delegated the PClassMapping roles to 
 * another instance. This implementation is dedicated to GenClass.
 * 
 * @author S.Chassande-Barrioz
 */
public class SpeedoGenClassHome extends SpeedoHomeImpl {

    PClassMapping pcm;
    
    public SpeedoGenClassHome(PClassMapping _pcm, 
            TransactionalPersistenceManager _tpm,
            ProxyManagerFactory _pmf) {
        super(_tpm, _pmf);
        this.pcm = _pcm;
    }
    
    // IMPLEMENTATION OF THE SpeedoHome INTERFACE //
    //--------------------------------------------//

    public boolean isDetachable() {
        return false;
    }

    public byte getVersioningStrategy() {
        return 0;
    }

    public Properties getClassProperties() {
        return null;
    }

        
    // IMPLEMENTATION OF THE PClassMapping INTERFACE //
    //-----------------------------------------------//
    
    public String getProjectName() {
        return pcm.getProjectName();
    }

    public PBinding createPBinding() throws PException {
        PBinding pb = pcm.createPBinding();
        pb.init(this);
        return pb;
        
    }

    public void init(PMappingCallback mapper, MetaObject metaclass)
            throws PException {
        pcm.init(mapper, metaclass);
    }

    public String getClassName() {
        return pcm.getClassName();
    }

    public PClassMapping getGenClassMapping()
            throws UnsupportedOperationException {
        return pcm.getGenClassMapping();
    }

    public PClassMapping getGenClassMapping(String fn)
            throws UnsupportedOperationException {
        return pcm.getGenClassMapping(fn);
    }

    public MetaObject getMetaInfo() {
        return pcm.getMetaInfo();
    }

    public PBinder getPBinder() {
        return pcm.getPBinder();
    }

    public PMapper getPMapper() {
        return pcm.getPMapper();
    }

    public PNameCoder getPNameCoder() throws UnsupportedOperationException {
        return pcm.getPNameCoder();
    }

    public PNameCoder getPNameCoder(String fn)
            throws UnsupportedOperationException {
        return pcm.getPNameCoder(fn);
    }

    public PNameCoder getClassPNameCoder() {
        return pcm.getClassPNameCoder();
    }

    public PNameIterator getPNameIterator(Object conn, boolean withSubType,
            boolean prefetching, Object txctx) throws PException {
        return pcm.getPNameIterator(conn, withSubType, prefetching, txctx);
    }

    public Iterator getPNameIterator(Object conn) throws PException {
        return pcm.getPNameIterator(conn, false, false, null);
    }

    public PType getPType() {
        return pcm.getPType();
    }

    public boolean isConform(String mappername) {
        return pcm.isConform(mappername);
    }

    public void setPBinder(PBinder pb) throws PException {
        pcm.setPBinder(pb);
    }

    public void configureRefFields(ReferenceConfigurator rc) throws PException,
            UnsupportedOperationException {
        pcm.configureRefFields(rc);
    }

    public boolean exist(PBinding pb, Object conn) throws PException {
        return pcm.exist(pb, conn);
    }

    public void read(PBinding pb, Object conn, PAccessor pa) throws PException {
        pcm.read(pb, conn, pa);
    }

    public void read(PBinding pb, Object conn, PAccessor pa, Object txctx)
            throws PException {
        pcm.read(pb, conn, pa, txctx);
    }

    public void write(PBinding pb, Object conn, PAccessor pa) throws PException {
        pcm.write(pb, conn, pa);
    }
    
    
    // IMPLEMENTATION OF THE Loggable INTERFACE //
    //------------------------------------------//
    
    public Logger getLogger() {
        return ((Loggable) pcm).getLogger();
    }

    public LoggerFactory getLoggerFactory() {
        return ((Loggable) pcm).getLoggerFactory();
    }

    public void setLogger(Logger logger) {
        ((Loggable) pcm).setLogger(logger);
    }

    public void setLoggerFactory(LoggerFactory loggerfactory) {
        ((Loggable) pcm).setLoggerFactory(loggerfactory);
    }

	public PClassMapping[] getSubPCMs() throws PException {
		return pcm.getSubPCMs();
	}

	public HashMap getAssociationTable() {
		return pcm.getAssociationTable();
	}

	public void addAssociation(PClassMapping targetClass, int[] indexes) {
		pcm.addAssociation(targetClass, indexes);		
	}

	public int[] getIndexesTable(PClassMapping targetClass) {
		return pcm.getIndexesTable(targetClass);
	}
}
