/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass.collection;

import org.objectweb.jorm.api.PIndexedElem;
import org.objectweb.jorm.api.PExceptionIO;
import org.objectweb.speedo.genclass.api.SpeedoGenClassProxy;
import org.objectweb.speedo.genclass.GenClassAccessor;

import java.util.List;
import java.util.Collection;
import java.util.ListIterator;
import java.util.Iterator;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class ListAccessor extends CollectionAccessor implements List{
    /**
     * Indicates the number of elements existing on the data support. All
     * element added after this index must be marked as CREATED.
     */
	protected int loadedSize = -1;

    public ListAccessor(SpeedoGenClassProxy jdoProxy) {
        super(jdoProxy);
    }
	public void workingSetClosed() {
		super.workingSetClosed();
		//Update the loaded size after the data support flushing
		loadedSize = elements.size();
	}


    public String toString() {
        return "ListAccessor: id=" + jdoProxy.jdoGetGenClassId();
    }

    public PIndexedElem createPIndexedElem(GenClassAccessor gca) {
        return new ListElem(gca);
    }

    public void paSetNbElem(int nbelem) {
        super.paSetNbElem(nbelem);
        loadedSize = 0;
    }

    public void paAdd(PIndexedElem elem, Object conn) throws PExceptionIO {
        super.paAdd(elem, conn);
        loadedSize++;
    }

    public boolean add(Object o) {
        if (super.add(o)) {
            int idx = elements.size() - 1;
            ListElem le = (ListElem) elements.get(idx);
            le.setIndex(new Integer(idx));
            return true;
        } else {
            return false;
        }
    }

    public boolean addAll(int i, Collection collection) {
        if (collection == null) {
            return false;
        }
        int size = elements.size();
        if (size > i) {
            Iterator it = collection.iterator();
            int j = i;
            while(it.hasNext()) {
                add(j, it.next());
                j++;
            }
            return true;
        } else {
            throw new IndexOutOfBoundsException("Try to insert at the position "
                    + i + " whereas there is only " + size + " elements.");
        }
    }

    public Object get(int i) {
	    ListElem le = (ListElem) elements.get(i);
	    if (le.getElemStatus() == PIndexedElem.ELEM_DELETED) {
		    throw new java.lang.ArrayIndexOutOfBoundsException(i);
	    }
	    return le.getElement();
    }

	public Object set(int i, Object o) {
		ListElem le = (ListElem) elements.get(i);
		if (le.getElemStatus() == PIndexedElem.ELEM_DELETED) {
			throw new java.lang.ArrayIndexOutOfBoundsException(i);
		}
		Object res = le.getElement();
		le.setStatus(PIndexedElem.ELEM_MODIFIED);
		le.setElement(o);
		return res;
	}

    public void add(int i, Object o) {
        int size = elements.size();
        if (size > i) {
            Object toAdd = o;
            Object tmp = o;
            int j = i;
            ListElem le;
	        //insert the new and shift others
            while(j < size) {
                le =(ListElem) elements.get(j);
                tmp = le.element;
                le.element = toAdd;
                le.setStatus((j < loadedSize
                        ? PIndexedElem.ELEM_MODIFIED
                        : PIndexedElem.ELEM_CREATED));
                toAdd = tmp;
                j++;
            }
	        //create a new entry for the shifted element (or the added if i = size)
            le = (ListElem) createPIndexedElem();
            le.setStatus((j < loadedSize
                    ? PIndexedElem.ELEM_MODIFIED
                    : PIndexedElem.ELEM_CREATED));
            le.setIndex(new Integer(size));
            elements.add(le);
        } else {
            throw new IndexOutOfBoundsException("Try to insert at the position "
                    + i + " whereas there is only " + size + " elements.");
        }
    }

    public Object remove(int i) {
        return null;
    }

    public int indexOf(Object o) {
        return 0;
    }

    public int lastIndexOf(Object o) {
        return 0;
    }

    public ListIterator listIterator() {
        return null;
    }

    public ListIterator listIterator(int i) {
        return null;
    }

    public List subList(int i, int i1) {
        return null;
    }
}
