/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.genclass.map;

import org.objectweb.speedo.genclass.GenClass;
import org.objectweb.speedo.mim.api.SpeedoAccessor;
import org.objectweb.speedo.mim.api.SpeedoProxy;
import org.objectweb.speedo.pm.api.ProxyManager;

import java.util.Map;
import java.util.Collection;
import java.util.Set;
import java.util.HashMap;

public class MapImpl extends GenClass implements Map {

    /**
     * Instantiates a new map.
     */
    public MapImpl() {
        super();
		accessor = (MapAccessor) createAccessor();
    }

    // ------------------------------------------------------------------------
    // IMPLEMENTATION OF THE Collection INTERFACE
    // ------------------------------------------------------------------------

    public int size() {
        if (!jdoIsActive) {
            return accessor.size();
        } else {
			MapAccessor ca = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ca.size();
        }
    }

    public boolean isEmpty() {
        if (!jdoIsActive) {
            return accessor.isEmpty();
        } else {
			MapAccessor ca = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ca.isEmpty();
        }
    }

    public boolean containsKey(Object key) {
        if (!jdoIsActive) {
            return accessor.containsKey(key);
        } else {
			MapAccessor ca = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ca.containsKey(key);
        }
    }

    public boolean containsValue(Object value) {
        if (!jdoIsActive) {
            return accessor.containsValue(value);
        } else {
			MapAccessor ca = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ca.containsValue(value);
        }
    }

    public Object get(Object key) {
        if (!jdoIsActive) {
            return accessor.get(key);
        } else {
			MapAccessor ca = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ca.get(key);
        }
    }

    public Object put(Object key, Object value) {
        if (!jdoIsActive) {
            return accessor.put(key, value);
        } else {
			MapAccessor ca = (MapAccessor) getSpeedoHome().writeIntention(this, null);
            return ca.put(key, value);
        }
    }

    public Object remove(Object key) {
        if (!jdoIsActive) {
            return accessor.remove(key);
        } else {
			MapAccessor ca = (MapAccessor) getSpeedoHome().writeIntention(this, null);
            return ca.remove(key);
        }
    }

    public void putAll(Map t) {
        if (!jdoIsActive) {
            accessor.putAll(t);
        } else {
			MapAccessor ca = (MapAccessor) getSpeedoHome().writeIntention(this, null);
            ca.putAll(t);
        }
    }

    public void clear() {
        if (!jdoIsActive) {
            accessor.clear();
        } else {
			MapAccessor ca = (MapAccessor) getSpeedoHome().writeIntention(this, null);
            ca.clear();
        }
    }

    public Set keySet() {
        if (!jdoIsActive) {
            return accessor.keySet();
        } else {
			MapAccessor ca = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ca.keySet();
        }
    }

    public Collection values() {
        if (!jdoIsActive) {
            return accessor.values();
        } else {
			MapAccessor ca = (MapAccessor) getSpeedoHome().writeIntention(this, null);
            return ca.values();
        }
    }

    public Set entrySet() {
        if (!jdoIsActive) {
            return accessor.entrySet();
        } else {
			MapAccessor ca = (MapAccessor) getSpeedoHome().readIntention(this, null);
            return ca.entrySet();
        }
    }

    // IMPLEMENTATION OF THE SpeedoProxy INTERFACE //
    //---------------------------------------------//

    public void setGCValue(Object o) {
        if (jdoIsActive) {
			MapAccessor ca = (MapAccessor) getSpeedoHome().writeIntention(this, null);
            ca.clear();
            if (o != null) {
                ca.putAll(((Map) o));
            }
        } else {
            accessor.clear();
            if (o != null) {
                accessor.putAll(((Map) o));
            }
        }
    }

	MapAccessor accessor;

	public SpeedoAccessor getReferenceAccessor() {
		return accessor;
	}
	public void setReferenceAccessor(SpeedoAccessor refAcc) {
		accessor = (MapAccessor) refAcc;
	}
    public SpeedoAccessor createAccessor() {
        return new MapAccessor(this);
    }

    public Object createGenClass() {
        return new HashMap();
    }

}
