/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.generation.api;

import org.apache.tools.ant.taskdefs.Javac;
import org.apache.tools.ant.types.Path;
import org.objectweb.speedo.metadata.SpeedoMetaInfo;
import org.objectweb.speedo.naming.lib.NamingManagerFactory;
import org.objectweb.util.monolog.api.LoggerFactory;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

/**
 * This class groups all parameter of the Speedo generator/compiler/enhancer.
 *
 * @author S.Chassande-Barrioz
 */
public class SpeedoCompilerParameter {

    /**
     * The logging configuration file
     */
	public String logPropFile = null;

    /**
     * The logger factory instance
     */
	public LoggerFactory loggerFactory = null;

    /**
     * The speedo configuration file
     */
	public String confFile = null;

	/**
	 * The jorm project name. The default value is an empty string (not nul)
	 */
	public String projectName = "";

	/**
	 * The jorm mapper name. The default value is null. This field MUST be set
	 */
	public String mapperName = null;

	/**
	 * indicates  if the generated java file must be kept or not
	 */
    public boolean keepsrc = true;

	/**
	 * the location of the user .class files
	 * the location where the generated files will be produced
	 */
	public String output;

	/**
	 * The default source directory
	 */
	public String input;

    public Collection jormclasspath = null;
    public Path speedoclasspath = null;

    /**
     * indicates if the .pd files must be generated
     */
	public boolean generateNeededJormFile = false;

	/**
	 * The output directory of the .pd files
	 */
    public String pdoutput;

	/**
	 * The Speedo meta information
	 */
    public SpeedoMetaInfo smi = new SpeedoMetaInfo();

    /**
     * the classpath to compile. must include Jorm, Speedo, user classes.
     */
    public Path classpath = null;

    /**
     * The list of .jdo file locations. The location is relative to the value
     * of the 'jdoDir' variable.
     */
	public Collection jdo = null;

	/**
	 * The base directory of the location of .jdo file
	 */
	public String jdoDir = null;

	/**
	 * The list of .pd file locations. The location is relative to the value
	 * of the 'jormDir' variable.
	 */
	public Collection jorm = null;

	/**
	 * The base directory of the location of .pd file
	 */
    public String jormDir = null;

	/**
	 * The list of .class file locations of the aware class. The location is
	 * relative to the value of the 'jormDir' variable.
	 */
    public Collection awareFiles = null;

	/**
	 * The base directory of the location of .class of aware class
	 */
    public String awareFilesDir = null;

	/**
	 * list of DTDLocation instance
	 */
	public ArrayList dtdLocations = new ArrayList();

	/**
	 * The compiler to use
	 */
	public Javac javac = null;

    /**
     * The NamingManagerFactory instance to use. It permits to build the jorm
     * meta information.
     */
	public NamingManagerFactory nmf = new NamingManagerFactory();

	public Map getXmldescriptor() {
		return smi.xmldescriptor;
	}

	public void setXmldescriptor(Map xmldescriptor) {
		smi.xmldescriptor = xmldescriptor;
	}

}
