/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */
package org.objectweb.speedo.generation.enhancer;

import org.objectweb.asm.ClassVisitor;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import java.util.List;

public class ClassInterfaceAdder extends LoggedClassAdapter {

    private List itfs;

    public ClassInterfaceAdder(ClassVisitor classVisitor,
                               List itfs,
                               Logger logger) {
        super(classVisitor, logger);
        this.itfs = itfs;
    }

    public void visit(final int access,
                      final String name,
                      final String superName,
                      final String[] interfaces,
                      final String sourceFile) {
        // just stores the interfaces implemented by the visited class
        if (interfaces != null) {
            for (int i = 0; i < interfaces.length; ++i) {
                if (!itfs.contains(interfaces[i])) {
                    if (debug) {
                        logger.log(BasicLevel.DEBUG,
                                "Add the interface '" + interfaces[i] + "'");
                    }
                    itfs.add(interfaces[i]);
                }
            }
        }
        String[] newInterfaces = (String[]) itfs.toArray(new String[itfs.size()]);
        cv.visit(access, name, superName, newInterfaces, sourceFile);
    }
}
