/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.generation.enhancer;

import org.objectweb.asm.Attribute;
import org.objectweb.asm.ClassVisitor;
import org.objectweb.asm.Constants;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.util.monolog.api.Logger;

/**
 * This visistor removes the persistent fields of a persistent class.
 * 
 * @author S.Chassande-Barrioz
 */
public class PersistentFieldRemover extends LoggedClassAdapter {

	SpeedoClass clazz;

	public PersistentFieldRemover(final ClassVisitor classVisitor,
								  final SpeedoClass jdoClass) {
		super(classVisitor);
		this.clazz = jdoClass;
	}

	public PersistentFieldRemover(final ClassVisitor classVisitor,
								  final SpeedoClass jdoClass,
								  final Logger logger) {
		super(classVisitor, logger);
		this.clazz = jdoClass;
	}

	public void visitField(final int access,
						   final String name,
						   final String desc,
						   final Object value,
						   final Attribute attrs) {
		if (((access & Constants.ACC_STATIC) != 0)
				|| clazz.jdoField.get(name)==null) {
			super.visitField(access, name, desc, value, attrs);
		}
	}
}
