/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.generation.generator.proxy;

import org.apache.velocity.app.Velocity;
import org.apache.velocity.context.Context;
import org.objectweb.jorm.lib.JormPathHelper;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.ClassRef;
import org.objectweb.jorm.metainfo.api.GenClassMapping;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.ReferenceMapping;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.generation.generator.api.SpeedoGenerationException;
import org.objectweb.speedo.generation.generator.lib.SpeedoGenerator;
import org.objectweb.speedo.mapper.lib.Object2StringSerializer;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.speedo.tools.StringReplace;
import org.objectweb.util.monolog.wrapper.velocity.VelocityLogger;

import java.io.File;
import java.io.FileWriter;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

/**
 * This class is used to generate the file proxy.
 * This file will represent the persistent instance.
 * @author	S. Chassande-Barrioz
 */
public class ProxyGenerator
	extends SpeedoGenerator {

    public final static String LOGGER_NAME
            = SpeedoProperties.LOGGER_NAME + ".generation.generator.proxy";

    public final static String TEMPLATE_NAME = TEMPLATE_DIR + ".proxy.Proxy";

    // IMPLEMENTATION OF THE GeneratorComponent INTERFACE //
    //----------------------------------------------------//

    public boolean init() throws SpeedoException {
        logger = scp.loggerFactory.getLogger(LOGGER_NAME);
	    return !scp.getXmldescriptor().isEmpty();
    }


    // IMPLEMENTATION OF THE VelocityGenerator INTERFACE //
    //---------------------------------------------------//

    /**
     * This method generates the new file.
     *
     * @param sClass is the SpeedoClass
     * @param fileName name of the new file.
     * @exception org.objectweb.speedo.generation.generator.api.SpeedoGenerationException if there is a problem during writing
     * the new file.
     */
    public void generate(SpeedoClass sClass, String fileName)
            throws SpeedoException {
        computeTemplate(TEMPLATE_NAME.replace('.', '/') + ".vm");
        try {
            Context ctx = getContext(sClass);
            FileWriter fw = new FileWriter(fileName);
            ve.setProperty(Velocity.RUNTIME_LOG_LOGSYSTEM, 
                    new VelocityLogger(logger));
            template.merge(ctx, fw);
            fw.flush();
            fw.close();
        } catch (Exception e) {
            throw new SpeedoGenerationException(
                    "Error during the generation of the file " + fileName, e);
        }
    }

    // PRIVATE METHODS //
    //-----------------//

    /**
     * This method initialises the Velocity context.
     *
     * @param jdoClass : the jdoClass which represents the class to generate.
     * @return a Velocity context.
     */
    protected Context getContext(SpeedoClass jdoClass) throws SpeedoException {
		Context ctx = super.getContext(jdoClass);

        //class declaration
        ctx.put("signatureClass", jdoClass.signature);
        ctx.put("serialversionuid", jdoClass.VersionUID + "L");

        //Define the 'pnameHints' attribute.
        String pnamehints = scp.nmf.getNamingManager(jdoClass).getPNameHints(
                        jdoClass, getClassNameDef(jdoClass.jormclass));
        ctx.put("pnameHints", pnamehints);
        return ctx;
    }
}
