/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.generation.jorm;

import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.ClassMapping;
import org.objectweb.jorm.metainfo.api.Mapping;
import org.objectweb.jorm.metainfo.api.PrimitiveElementMapping;
import org.objectweb.jorm.metainfo.api.PrimitiveElement;
import org.objectweb.jorm.metainfo.api.CommonClassMapping;
import org.objectweb.jorm.metainfo.api.GenClassMapping;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.api.PException;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.metadata.SpeedoElement;
import org.objectweb.speedo.api.SpeedoException;

/**
 * This interface defines a builder of the mapping part of the Jorm meta
 * information. It must be implemented for each mapper.
 *
 * @author S.Chassande-Barrioz
 */
public interface JormMIMappingBuilder {

    /**
     * It builds a ClassMapping, assignes it to the mapping and builds
     * mapping structure for the class (RdbTable, directory name, ...).
     * @param clazz is the Jorm meta object representing the class which the
     * ClassMapping must be built.
     * @param mapping is the Mapping instance which will host the ClassMapping.
     * @return the ClassMapping instance built by the method (never null).
     * @throws PException if it is not possible to build the ClassMapping
     */
    ClassMapping createClassMapping(Class clazz,
                                    SpeedoClass sc,
                                    Mapping mapping)
            throws PException, SpeedoException ;

    /**
     * It finishes the filling of a ClassMapping (Inheritance for example).
     * @throws PException if it is not possible to build the ClassMapping
     */
    void finalizeClassMapping(SpeedoClass sc,
                              String projectName,
                              String mapperName)
            throws PException, SpeedoException ;

    /**
     * It builds a GenClassMapping, assignes it to the mapping and builds
     * mapping structure for the class (RdbTable, directory name, ...).
     * @param gcr is the Jorm meta object representing the gen class which the
     * GenClassMapping must be built.
     * @param mapping is the Mapping instance which will host the GenClassMapping.
     * @return the GenClassMapping instance built by the method (never null).
     * @throws PException if it is not possible to build the GenClassMapping
     */
    GenClassMapping createGenClassMapping(GenClassRef gcr,
                                          SpeedoField sf,
                                          Mapping mapping)
            throws PException, SpeedoException ;

    /**
     * It creates the mapping of a primitive field.
     * @param pe is the Jorm meta object representing a primitive field
     * @param cm is the MappingStructure which will host the mapping of the
     * field
     * @return a PrimitiveElementMapping corresponding to the given primitive
     * field.
     * @throws PException if it is not possible to build the mapping of the
     * primitive field.
     */
    PrimitiveElementMapping createFieldMapping(PrimitiveElement pe,
                                               SpeedoElement selem,
                                               CommonClassMapping cm)
            throws PException, SpeedoException ;

    /**
     * It creates the mapping of a name def (reference or identifier).
     * @param cm is the MappingStructure which will host the mapping of the
     * fields of the name def
     * @param nd is the NameDef which must be mapped.
     * @throws PException if it is not possible to build the mapping of the
     * field.
     */
    void createNameDefMapping(CommonClassMapping cm,
                              NameDef nd,
                              SpeedoClass sc,
                              boolean isIdentifier,
                             boolean isInGenClass)
            throws PException, SpeedoException;


}
