/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.generation.mivisitor;

import org.objectweb.speedo.generation.api.SpeedoCompilerParameter;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.metadata.SpeedoXMLDescriptor;
import org.objectweb.speedo.metadata.SpeedoPackage;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.metadata.SpeedoExtension;
import org.objectweb.speedo.metadata.SpeedoElement;

import java.util.Iterator;

/**
 *
 * @author S.Chassande-Barrioz
 */
public class MetaInfoVisitorImpl extends AbstractMetaInfoVisitor {

	/**
	 * builds a MetaInfoVisitor which is the last of the chain
	 */
	public MetaInfoVisitorImpl() {
	}

	/**
	 * builds a MetaInfoVisitor which is the last of the chain
	 */
	public MetaInfoVisitorImpl(MetaInfoVisitor mim) {
		super(mim);
	}

    public void visitCompilerParameter(SpeedoCompilerParameter scp) throws SpeedoException {
        super.visitCompilerParameter(scp);
        Iterator xmlIt = scp.getXmldescriptor().values().iterator();
        while(xmlIt.hasNext()) {
            SpeedoXMLDescriptor xml = (SpeedoXMLDescriptor) xmlIt.next();
            visitXml(xml);
        }
    }

    public void visitXml(SpeedoXMLDescriptor xml) throws SpeedoException {
        super.visitXml(xml);
        Iterator packIt = xml.jdoPackage.values().iterator();
        while(packIt.hasNext()) {
            SpeedoPackage pack = (SpeedoPackage) packIt.next();
            visitPackage(pack);
        }
    }

    public void visitPackage(SpeedoPackage sp) throws SpeedoException {
        super.visitPackage(sp);
        visitExtension(sp);
        Iterator classIt = sp.jdoClass.values().iterator();
        while(classIt.hasNext()) {
            SpeedoClass clazz = (SpeedoClass) classIt.next();
            visitClass(clazz);
        }
    }

    public void visitClass(SpeedoClass sc) throws SpeedoException {
        super.visitClass(sc);
        visitExtension(sc);
        Iterator fieldIt = sc.jdoField.values().iterator();
        while(fieldIt.hasNext()) {
            SpeedoField sf = (SpeedoField) fieldIt.next();
            visitField(sf);
        }
    }

    public void visitField(SpeedoField sf) throws SpeedoException {
        visitExtension(sf);
        super.visitField(sf);
    }

    private void visitExtension(SpeedoElement se) throws SpeedoException {
        for(int i=0; i<se.jdoExtension.size(); i++) {
            super.visitExtension((SpeedoExtension) se.jdoExtension.get(i));
        }
    }

}
