/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.log;

import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.Monolog;

/**
 * Logging system initialisation.
 * By default, the loggerFactory class is  <code>org.objectweb.util.monolog.wrapper.log4j.MonologLoggerFactory</code>
 * @author	S.Chassande-Barrioz
 */
public class Log
        implements LoggerFactory, LogAttributes {

    /**
     * Default name of the logger factory
     */
    public final static String DEFAULT_LOGGER_FACTORY =
            "org.objectweb.util.monolog.wrapper.printwriter.LoggerImpl";

	private static LoggerFactory lastLF = null;

    public static LoggerFactory getLoggerFactory() {
        if (lastLF == null) {
	        lastLF = Monolog.initialize();
        }
        return lastLF;
    }

    private LoggerFactory delegate = null;
    private String pfn = null;

    // IMPLEMENTATION OF THE LogAttributes INTERFACE //
    //-----------------------------------------------//

    public String getPropertiesFileName() {
        return pfn;
    }

    public void setPropertiesFileName(String propfn) {
        if (pfn != null && pfn.equals(propfn))
            return;
        pfn = propfn;
        delegate = Monolog.getMonologFactory(propfn);
        lastLF = this;
    }

    // IMPLEMENTATION OF THE LoggerFactory INTERFACE //
    //-----------------------------------------------//

    public Logger getLogger(String s) {
        return delegate.getLogger(s);
    }

    public Logger getLogger(String s, String s1) {
        return delegate.getLogger(s1);
    }

    public String getResourceBundleName() {
        return delegate.getResourceBundleName();
    }

    public void setResourceBundleName(String s) {
        delegate.setResourceBundleName(s);
    }

    public Logger[] getLoggers() {
        return delegate.getLoggers();
    }
}
