/**
 * Speedo: an implementation of JDO compliant personality on top of JORM generic
 * I/O sub-system.
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Contact: speedo@objectweb.org
 *
 * Authors: S.Chassande-Barrioz.
 *
 */

package org.objectweb.speedo.metadata;

import org.objectweb.asm.Constants;

/**
 * Stores info on methods of a <code>SpeedoClass</code>
 * @author	S.Chassande-Barrioz
 */
public class CodeMethod {

    /**
     * Name of the method.If the method is a constructor the name of the method
     * is null.
     */
    public String name;

    /**
     * is the modifiers of the method
     * 'public final'
     */
    public String modifiers;

    /**
     * The declaraction of the method parameters.
     * 'String arg1, int arg2'
     */
    public String params;

    /**
     * The list of the exception thrown by the method
     */
    public String[] exceptions;

    /**
     * Indicates whether this method is a constructor
     */
    public boolean isConstructor;

    /**
     * Call to the super constructor
     */
    public String superClass;

    /**
     * Descriptor of the method (encoded form of its signature and return type)
     */
    public String methodDesc;


    /**
     * @param name is the ASM name of the
     * @param params is the declaraction of method parameters separated by
     * commas
     * @param superclass is a string representing the call to the super method.
     * @param isConstr indicates if the method is a constructor.
     * @param modifiers is the ASM code representing the method modifiers
     * @param exceptions is the list of exceptions thrown by the method
     */
    public CodeMethod(String name,
                      String params,
                      String superclass,
                      boolean isConstr,
                      int modifiers,
                      String[] exceptions) {
        if ((modifiers & Constants.ACC_PUBLIC) != 0)
            this.modifiers = "public";
        else if ((modifiers & Constants.ACC_PRIVATE) != 0)
            this.modifiers = "protected";
        else if ((modifiers & Constants.ACC_PROTECTED) != 0)
            this.modifiers = "protected";

        if ((modifiers & Constants.ACC_FINAL) != 0)
            this.modifiers += " final";
        if ((modifiers & Constants.ACC_STATIC) != 0)
            this.modifiers += " static";
        if ((modifiers & Constants.ACC_ABSTRACT) != 0)
            this.modifiers += " abstract";

        this.isConstructor = isConstr;
        this.name = (isConstructor ? null : name);
        this.params = params;
        this.superClass = superclass;
        this.exceptions = exceptions;
    }
}
