/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.naming.lib;

import org.objectweb.asm.Type;
import org.objectweb.jorm.api.PException;
import org.objectweb.jorm.facility.naming.basidir.BasidBinder;
import org.objectweb.jorm.facility.naming.basidir.BasidName;
import org.objectweb.jorm.metainfo.api.Class;
import org.objectweb.jorm.metainfo.api.GenClassRef;
import org.objectweb.jorm.metainfo.api.CommonClassMapping;
import org.objectweb.jorm.metainfo.api.Manager;
import org.objectweb.jorm.metainfo.api.MetaObject;
import org.objectweb.jorm.metainfo.api.NameDef;
import org.objectweb.jorm.metainfo.api.Reference;
import org.objectweb.jorm.metainfo.api.TypedElement;
import org.objectweb.jorm.naming.api.PBinder;
import org.objectweb.jorm.naming.api.PExceptionNaming;
import org.objectweb.jorm.naming.api.PName;
import org.objectweb.jorm.naming.api.PNameCoder;
import org.objectweb.jorm.naming.lib.CTHelper;
import org.objectweb.jorm.type.api.PType;
import org.objectweb.speedo.api.SpeedoException;
import org.objectweb.speedo.api.SpeedoProperties;
import org.objectweb.speedo.generation.jorm.JormMIMappingBuilder;
import org.objectweb.speedo.generation.lib.NamingRules;
import org.objectweb.speedo.mapper.api.JormFactory;
import org.objectweb.speedo.metadata.SpeedoClass;
import org.objectweb.speedo.metadata.SpeedoExtension;
import org.objectweb.speedo.metadata.SpeedoField;
import org.objectweb.speedo.metadata.SpeedoIdentity;
import org.objectweb.speedo.naming.api.MIBuilderHelper;
import org.objectweb.speedo.naming.api.NamingManager;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

/**
 * Is a manager of identifier naming compised of a single visible field.
 *
 * @author S.Chassande-Barrioz
 */
public class UserIdSingleNamingManager
	extends NamingManagerHelper
	implements NamingManager {

	private final static String SINGLE_USER_ID = "suid";

    private int getCodingType(String fn, MetaObject mo) {
        TypedElement te = null;
        if (mo instanceof Class) {
            te = ((Class) mo).getTypedElement(fn);
        } else if (mo instanceof GenClassRef) {
            te = ((GenClassRef) mo).getHiddenField(fn);
        } else {
            return -1;
        }
        return CTHelper.ptc2ct(te.getType().getTypeCode());
    }

	protected String getName() {
		return SINGLE_USER_ID;
	}
    // IMPLEMENTATION OF THE METHOD FROM THE NamingManager INTERFACE //
    //---------------------------------------------------------------//

    public Object encode(PName pn) throws PException {
        if (pn instanceof BasidName) {
            return pn.getPNameManager().getPType().getJormName()
                    + SEP + pn.encodeString();
        }
        return null;
    }

    public PName decode(PNameCoder pnc,
                        Object oid,
                        java.lang.Class clazz,
                        JormFactory jf) throws PException {
        if (oid instanceof String) {
            String stroid = (String) oid;
            int idx = stroid.indexOf(SEP);
            if (pnc != null) {
				if (pnc.codingSupported(PNameCoder.CTCOMPOSITE)) {
					//The pnc is not a BasidBinder, then the oid cannot be managed
					return null;
				}
				if (idx != -1) {
					//The oid contains the class name
					return pnc.decodeString(stroid.substring(idx + SEP.length()));
				} else {
					//The oid must decoded directly
					return pnc.decodeString(stroid);
				}
            }
			//No pnc specified
			if (idx == -1) {
				//The oid cannot be managed
				return null;
			}
			//The oid contains the class name
			String fqcn = stroid.substring(0, idx);
			ClassLoader cl = jf.getClass().getClassLoader();
			if (cl == null) {
				cl = ClassLoader.getSystemClassLoader();
			}
			try {
				pnc = jf.getPNamingContext(fqcn, cl);
			} catch (Exception e) {
				return null;
			}
			if (pnc == null
				|| pnc.codingSupported(PNameCoder.CTCOMPOSITE)) {
				return null;
			}
			return pnc.decodeString(stroid.substring(idx + SEP.length()));
        } else if (pnc instanceof BasidBinder) {
            switch (((BasidBinder) pnc).getCodingType()) {
            case PNameCoder.CTCHAR:
                return pnc.decodeChar(((Character) oid).charValue());
            case PNameCoder.CTBYTE:
                return pnc.decodeByte(((Byte) oid).byteValue());
            case PNameCoder.CTSHORT:
                return pnc.decodeShort(((Short) oid).shortValue());
            case PNameCoder.CTINT:
                return pnc.decodeInt(((Integer) oid).intValue());
            case PNameCoder.CTLONG:
                return pnc.decodeLong(((Long) oid).longValue());
            case PNameCoder.CTOCHAR:
                return pnc.decodeOchar((Character) oid);
            case PNameCoder.CTOBYTE:
                return pnc.decodeObyte((Byte) oid);
            case PNameCoder.CTOSHORT:
                return pnc.decodeOshort((Short) oid);
            case PNameCoder.CTOINT:
                return pnc.decodeOint((Integer) oid);
            case PNameCoder.CTOLONG:
                return pnc.decodeOlong((Long) oid);
            case PNameCoder.CTSTRING:
                return pnc.decodeString((String) oid);
            case PNameCoder.CTDATE:
                return pnc.decodeDate((Date) oid);
            case PNameCoder.CTCHARARRAY:
                return pnc.decodeCharArray((char[]) oid);
            case PNameCoder.CTBYTEARRAY:
                return pnc.decode((byte[]) oid);
            default:
                throw new PExceptionNaming("Unmanaged coding type: "
                        + ((BasidBinder) pnc).getCodingType());
            }
        } else {
            //oid
            return null;
        }
    }

    public boolean canManage(SpeedoClass sc) {
        if (sc.identityType == SpeedoIdentity.USER_ID) {
            ArrayList idFields = new ArrayList();
            Iterator fieldsIt = sc.jdoField.values().iterator();
            while (fieldsIt.hasNext()) {
                SpeedoField sp = (SpeedoField) fieldsIt.next();
                if (sp.primaryKey) {
                    idFields.add(sp);
                }
            }
            String objectidClass = sc.objectidClass;
            if (sc.generateObjectId()) {
                objectidClass = NamingRules.generatedObjectIdName(sc.getFQName());
            }
            return objectidClass == null || objectidClass.length() == 0;
        }
        return false;
    }

    public boolean canProvidePBinder(Object hints, ClassLoader classLoader) {
		if(!super.canProvidePBinder(hints, classLoader)) {
			return false;
		}
		String ctStr = getBinderClassNameFromHints(hints, SINGLE_USER_ID);
		if (ctStr == null) {
			return false;
		}
		try {
			Short.parseShort(ctStr);
			return true;
		} catch (NumberFormatException e) {
			return false;
		}
	}

    public boolean canProvidePNamingContext(Object hints, ClassLoader classLoader) {
		return getPNCClassNameFromHints(hints, SINGLE_USER_ID) != null;
    }

    public PBinder getPBinder(String className, String hints, ClassLoader cl, byte mappingStructureRule, Map cn2binder, Map cn2pnc) {
		String ctStr = getBinderClassNameFromHints(hints, SINGLE_USER_ID);
        return new BasidBinder(Short.parseShort(ctStr));
    }

	public void getJormNamingConfig(NameDef nd,
									SpeedoClass targetClass,
									MetaObject sourceMO,
									String key,
									Properties result) {
		StringBuffer sb = new StringBuffer();
		sb.append(SINGLE_USER_ID);
		sb.append(HINTS_SEP);
		int ct = getCodingType(nd.getFieldName(), sourceMO);
		sb.append(ct);
		sb.append(HINTS_SEP);
		String className = targetClass.getFQName();
		SpeedoClass ancestor = targetClass.getAncestor();
		if (sourceMO instanceof GenClassRef //For a Genclass ref or identifier
			|| (ancestor == null //No inheritance
				&& targetClass.jormclass.getSubClasses().isEmpty())) {
			sb.append(ct);
		} else {
			if (ancestor == null) {
				ancestor = targetClass;
			}
			sb.append(NamingRules.kfpncName(ancestor.getFQName()));
			className = ancestor.getFQName();
		}
		sb.append(HINTS_SEP);
		sb.append(className);
		result.setProperty(key, sb.toString());
    }

    public String getPNameHints(SpeedoClass sc, NameDef nd) {
        //provide the field value as Object format
        String fn = nd.getFieldName();
        int tc = sc.jormclass.getTypedElement(fn).getType().getTypeCode();
		fn = "jdoReferenceState." + fn;
        switch (tc) {
        case PType.TYPECODE_CHAR:
            return "new Character(" + fn + ")";
        case PType.TYPECODE_BYTE:
            return "new Byte(" + fn + ")";
        case PType.TYPECODE_SHORT:
            return "new Short(" + fn + ")";
        case PType.TYPECODE_INT:
            return "new Integer(" + fn + ")";
        case PType.TYPECODE_LONG:
            return "new Long(" + fn + ")";
        default:
            return fn;
        }
    }

	public String getGCPNameHints(SpeedoClass sc, NameDef nd) {
		return "proxy.getPName()";
	}

    public void fillNameDef(MIBuilderHelper mibh,
                            Manager manager,
                            NameDef nd,
                            SpeedoClass tsc,
                            SpeedoClass ssc,
                            MetaObject mo,
                            Reference ref,
                            CommonClassMapping hcm,
                            JormMIMappingBuilder mb,
                            boolean isIdentifier,
                            boolean isInGenClass,
                            boolean createField,
                            Collection createdMOs) throws SpeedoException, PException {
        ArrayList idFields = new ArrayList();
        Iterator fieldsIt = tsc.jdoField.values().iterator();
        while (fieldsIt.hasNext()) {
            SpeedoField sp = (SpeedoField) fieldsIt.next();
            if (sp.primaryKey) {
                idFields.add(sp);
            }
        }
        if (idFields.size() == 0) {
            throw new SpeedoException(
                    mibh.getErrorMessage(tsc, mo, ref)
                    + "no primary key field and no object id defined");
        }
        if (idFields.size() > 1) {
            String fn = "[";
            String sep = "";
            for (int i = 0; i < idFields.size(); i++) {
                fn += sep + ((SpeedoField) idFields.get(i)).name;
                sep = ",";
            }
            fn += "]";
            throw new SpeedoException(
                    mibh.getErrorMessage(tsc, mo, ref)
                    + "there are several primary key fields "
                    + fn
                    + " and no an object id class.");
        }
        SpeedoField sf = (SpeedoField) idFields.get(0);
        String clafn = mibh.getNameDefFieldPrefix(
                ref, isIdentifier, isInGenClass) + sf.name;
        if (createField) {
            int size = PType.NOSIZE;
            SpeedoExtension se = sf.getExtension(SpeedoProperties.VENDOR_NAME,SpeedoProperties.SIZE);
            if (se != null) {
                try {
                    size = Integer.parseInt(se.value);
                } catch (NumberFormatException e) {
                    //The error will occur during the parsing of the primitive
                    // field in the referenced class
                }
            }
            int scale = PType.NOSIZE;
            se = sf.getExtension(SpeedoProperties.VENDOR_NAME,SpeedoProperties.SCALE);
            if (se != null) {
                try {
                    scale = Integer.parseInt(se.value);
                } catch (NumberFormatException e) {
                    //The error will occur during the parsing of the primitive
                    // field in the referenced class
                }
            }
            mibh.createNameDefField(mo, clafn,
                    mibh.getPrimitivePType(Type.getType(sf.desc)), size, scale);
        }
        nd.setFieldName(clafn);
        if (createField && (mb != null)) {
            mb.createNameDefMapping(hcm, nd, ssc, isIdentifier, isInGenClass);
        }
    }
}
