/**
 * Copyright (C) 2001-2004 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.speedo.usercache.api;

/**
 * Defines an user view of the cache of persistent objects
 * 
 * @author S.Chassande-Barrioz
 */
public interface UserCacheView {

	/**
	 * Look for a persistent instance with an user identifier and a class 
	 * @param clazz is the class of the searched persistent instance 
	 * @param key is the user key of the searched persistent instance
	 * @return
	 */
	Object lookup(Class clazz, Object key);

	/**
	 * Register a persistent instance into the user view. The persistent 
	 * instance is bound with the couple (obj.getClass(), key). It means that
	 * entry are bound per class and par user key.
	 * 
	 * @param key is the user key of the persistent instance
	 * @param obj is the persistent instance. It cannot be null.
	 * 
	 * @return the previous associated persistent instance associated to the 
	 * specified user key. The null value is returned if no previous instance
	 * was registered.
	 */
	Object bind(Object key, Object obj);
	
	/**
	 * Register an entry with an new user key. The old registration is removed.
	 * @param oldkey is the old user key of the persistent instance
	 * @param newkey is the new user key of the persistent instance
	 * @param obj is the persistent instance. It cannot be null.
	 * 
	 * @return the previous associated persistent instance associated to the 
	 * new user key. The null value is returned if no previous instance
	 * was registered.
	 */
	Object rebind(Object oldkey, Object newkey, Object obj);
	
	/**
	 * Forces the eviction of an entry
	 * @param clazz is the class of the persistent instance 
	 * @param key is the user key of the persistent instance
	 * @return
	 */
	Object unbind(Class clazz, Object key);
	
	/**
	 * Try to auto clean the user cache view. 
	 */
	void clean();
}
